@@ -8,7 +8,7 @@ use JSON qw(from_json to_json);
 use Lemonldap::NG::Common::Crypto;
 use Lemonldap::NG::Portal::Main::Constants 'PE_OK';
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends 'Lemonldap::NG::Portal::2F::Register::Base';
 with 'Lemonldap::NG::Portal::Lib::2fDevices';
@@ -49,10 +49,6 @@ sub run {
 
     # Send a code to generic
     if ( $action eq 'sendcode' ) {
-
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         my $generic = $req->param('generic');
 
         unless ($generic) {
@@ -89,10 +85,6 @@ sub run {
 
     # Verification that user has a valid generic
     elsif ( $action eq 'verify' ) {
-
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         my $generic     = $req->param('generic');
         my $tokenid     = $req->param("token");
         my $genericcode = $req->param('genericcode');
@@ -139,8 +131,6 @@ sub run {
             )
           )
         {
-            $self->userLogger->notice( $self->prefix
-                  . "2f: registration of $genericname succeeds for $user" );
             return [
                 200,
                 [
@@ -158,9 +148,6 @@ sub run {
 
     elsif ( $action eq 'delete' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->userCanRemove;
@@ -171,8 +158,6 @@ sub run {
 
         if ( $self->del2fDevice( $req, $req->userData, $self->prefix, $epoch ) )
         {
-            $self->userLogger->notice(
-                $self->prefix . "2f: device deleted for $user" );
             return [
                 200,
                 [
