@@ -6,7 +6,7 @@ use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Common::Crypto;
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends 'Lemonldap::NG::Portal::2F::Register::Base';
 with 'Lemonldap::NG::Portal::Lib::2fDevices';
@@ -56,9 +56,6 @@ sub run {
     # Verification that user has a valid password
     if ( $action eq 'verify' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Check Password
         my $password       = $req->param('password');
         my $passwordverify = $req->param('passwordverify');
@@ -113,8 +110,6 @@ sub run {
             )
           )
         {
-            $self->userLogger->notice( $self->prefix
-                  . "2f: registration of password succeeds for $user" );
             return [
                 200,
                 [
@@ -132,9 +127,6 @@ sub run {
     }
     elsif ( $action eq 'delete' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->conf->{password2fUserCanRemoveKey};
