@@ -5,7 +5,7 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends qw(
   Lemonldap::NG::Portal::2F::Register::Base
@@ -41,9 +41,6 @@ sub run {
     # Verification that user has a valid TOTP app
     if ( $action eq 'verify' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Get form token
         my $token = $req->param('token');
         unless ($token) {
@@ -122,8 +119,6 @@ sub run {
             )
           )
         {
-            $self->userLogger->notice( $self->prefix
-                  . "2f: registration of $TOTPName succeeds for $user" );
             return [
                 200,
                 [
@@ -143,9 +138,6 @@ sub run {
     elsif ( $action eq 'getkey' ) {
         my ( $nk, $secret, $issuer ) = ( 0, '' );
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Read existing TOTP 2F
         my @totp2f =
           $self->find2fDevicesByType( $req, $req->userData, $self->type );
@@ -190,9 +182,6 @@ sub run {
     # Delete TOTP
     elsif ( $action eq 'delete' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->userCanRemove;
@@ -201,8 +190,6 @@ sub run {
           or return $self->p->sendError( $req,
             $self->prefix . '2f: "epoch" parameter is missing', 400 );
         if ( $self->del2fDevice( $req, $req->userData, $self->type, $epoch ) ) {
-            $self->userLogger->notice(
-                $self->prefix . "2f: device deleted for $user" );
             return [
                 200,
                 [
