@@ -6,7 +6,7 @@ use Mouse;
 use JSON qw(from_json to_json);
 use MIME::Base64 qw(encode_base64url decode_base64url);
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends qw(
   Lemonldap::NG::Portal::2F::Register::Base
@@ -39,7 +39,7 @@ sub run {
 
         # Read existing 2F device(s)
         my @alldevices = $self->find2fDevicesByType( $req, $req->userData );
-        my $challenge = $self->crypter->registrationChallenge;
+        my $challenge  = $self->crypter->registrationChallenge;
         $self->logger->debug(
             $self->prefix . "2f: registration challenge ($challenge)" );
         return [
@@ -92,9 +92,6 @@ sub run {
                     )
                   )
                 {
-                    $self->userLogger->notice( $self->prefix
-                          . "2f: registration of device $keyName succeeds for $user"
-                    );
                     return [
                         200,
                         [
@@ -202,9 +199,6 @@ sub run {
 
     elsif ( $action eq 'delete' ) {
 
-        $self->checkCsrf($req)
-            or return $self->p->sendError( $req, 'csrfError', 400 );
-
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->userCanRemove;
@@ -213,8 +207,6 @@ sub run {
           or return $self->p->sendError( $req,
             $self->prefix . '2f: "epoch" parameter is missing', 400 );
         if ( $self->del2fDevice( $req, $req->userData, $self->type, $epoch ) ) {
-            $self->userLogger->notice(
-                $self->prefix . "2f: device deleted for $user" );
             return [
                 200,
                 [
