@@ -7,15 +7,18 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_DONE
   PE_ERROR
   PE_LDAPCONNECTFAILED
-  PE_PP_CHANGE_AFTER_RESET
+  PE_PP_ACCOUNT_LOCKED
   PE_PP_PASSWORD_EXPIRED
+  PE_PP_CHANGE_AFTER_RESET
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.14';
 
 # Inheritance: UserDB::LDAP provides all needed ldap functions
-extends
-  qw(Lemonldap::NG::Portal::Auth::_WebForm Lemonldap::NG::Portal::Lib::LDAP);
+extends qw(
+  Lemonldap::NG::Portal::Lib::LDAP
+  Lemonldap::NG::Portal::Auth::_WebForm
+);
 
 sub init {
     my ($self) = @_;
@@ -35,14 +38,12 @@ has authnLevel => (
 
 sub authenticate {
     my ( $self, $req ) = @_;
-    unless ( $self->ldap ) {
-        return PE_LDAPCONNECTFAILED;
-    }
 
     # Set the dn unless done before
     unless ( $req->data->{dn} ) {
         if ( my $tmp = $self->getUser($req) ) {
-            $self->setSecurity($req);
+            eval { $self->setSecurity($req) };
+            $self->logger->warn($@) if ($@);
             return $tmp;
         }
     }
@@ -67,9 +68,17 @@ sub authenticate {
         # Security: never create session here
         return $res || PE_DONE;
     }
+
+    $self->validateLdap;
+
+    unless ( $self->ldap ) {
+        return PE_LDAPCONNECTFAILED;
+    }
+
     my $res =
-      $self->userBind( $req, $req->data->{dn},
+      $self->ldap->userBind( $req, $req->data->{dn},
         password => $req->data->{password} );
+    $self->setSecurity($req) if ( $res > PE_OK );
 
     # Remember password if password reset needed
     if (
@@ -78,7 +87,7 @@ sub authenticate {
             and $self->conf->{ldapAllowResetExpiredPassword} )
       )
     {
-        $req->data->{oldpassword} = $self->{password};
+        $req->data->{oldpassword} = $req->data->{password};    # Fix 2377
         $req->data->{noerror}     = 1;
         $self->setSecurity($req);
     }
@@ -88,18 +97,20 @@ sub authenticate {
 }
 
 sub authLogout {
-    PE_OK;
+    return PE_OK;
 }
 
-# Test LDAP connection before trying to bind
-sub userBind {
-    my $self = shift;
-    unless ($self->ldap
-        and $self->ldap->root_dse( attrs => ['supportedLDAPVersion'] ) )
-    {
-        $self->ldap( $self->newLdap );
-    }
-    return $self->ldap ? $self->ldap->userBind(@_) : undef;
+# Define which error codes will stop Combination process
+# @param res error code
+# @return result 1 if stop is needed
+sub stop {
+    my ( $self, $res ) = @_;
+
+    return 1
+      if ( $res == PE_PP_PASSWORD_EXPIRED
+        or $res == PE_PP_ACCOUNT_LOCKED
+        or $res == PE_PP_CHANGE_AFTER_RESET );
+    return 0;
 }
 
 1;
