@@ -9,9 +9,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_IDPCHOICE
   PE_OIDC_AUTH_ERROR
+  PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Auth
@@ -20,8 +21,9 @@ extends qw(
 
 # INTERFACE
 
-has opList => ( is => 'rw', isa => 'ArrayRef', default => sub { [] } );
-has path => ( is => 'rw', default => 'oauth2' );
+has opList      => ( is => 'rw', isa     => 'ArrayRef', default => sub { [] } );
+has path        => ( is => 'rw', default => 'oauth2' );
+has endpointsRe => ( is => 'rw' );
 
 use constant sessionKind => 'OIDC';
 
@@ -31,7 +33,7 @@ sub init {
     my $self = shift;
 
     return 0 unless ( $self->loadOPs and $self->refreshJWKSdata );
-    my @tab = ( sort keys %{ $self->oidcOPList } );
+    my @tab = ( sort keys %{ $self->opMetadata } );
     unless (@tab) {
         $self->logger->error("No OP configured");
         return 0;
@@ -41,17 +43,13 @@ sub init {
 
     foreach (@tab) {
         my $name = $_;
-        $name =
-          $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsDisplayName}
-          if $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsDisplayName};
-        my $icon = $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsIcon};
-        my $tooltip = $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsTooltip} || $name;
-        my $order = $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsSortNumber} // 0;
+        $name = $self->opOptions->{$_}->{oidcOPMetaDataOptionsDisplayName}
+          if $self->opOptions->{$_}->{oidcOPMetaDataOptionsDisplayName};
+        my $icon = $self->opOptions->{$_}->{oidcOPMetaDataOptionsIcon};
+        my $tooltip =
+          $self->opOptions->{$_}->{oidcOPMetaDataOptionsTooltip} || $name;
+        my $order = $self->opOptions->{$_}->{oidcOPMetaDataOptionsSortNumber}
+          // 0;
         my $img_src;
 
         if ($icon) {
@@ -71,20 +69,28 @@ sub init {
           };
     }
 
-    # If this module has already been loaded,
-    # skip routes to avoid a warning in conf
-    if ( !$self->p->loadedModules->{ ref($self) } ) {
-        $self->addRouteFromConf(
-            'Unauth',
-            oidcServiceMetaDataFrontChannelURI => 'alreadyLoggedOut',
-            oidcServiceMetaDataBackChannelURI  => 'backLogout',
-        );
-        $self->addRouteFromConf(
-            'Auth',
-            oidcServiceMetaDataFrontChannelURI => 'frontLogout',
-            oidcServiceMetaDataBackChannelURI  => 'backLogout',
-        );
-    }
+    my $portal = $self->conf->{portal};
+    my $re     = '^/' . $self->path . '/(?:' . join(
+        '|',
+        map {
+            my $s = $self->conf->{$_};
+            $s =~ s/^$portal\/*//;
+            $s =~ s/#PORTAL#\/*//;
+            $s
+        } ( qw(
+              oidcServiceMetaDataFrontChannelURI
+              oidcServiceMetaDataBackChannelURI
+            )
+        )
+    ) . ')(?:[\?/].*)?$';
+
+    $self->endpointsRe(qr/$re/);
+
+    $self->addRouteFromConf(
+        'Auth',
+        oidcServiceMetaDataFrontChannelURI => 'frontLogout',
+        oidcServiceMetaDataBackChannelURI  => 'backLogout',
+    );
 
     @list =
       sort {
@@ -100,6 +106,20 @@ sub init {
 
 sub extractFormInfo {
     my ( $self, $req ) = @_;
+    if ( $req->uri =~ $self->endpointsRe ) {
+        my $path = $req->path;
+        my $tmp  = $self->path;
+        $path =~ s#^/*\Q$tmp\E/*##o;
+        $path =~ s#/.*$##;
+        if ( $path eq $self->conf->{oidcServiceMetaDataBackChannelURI} ) {
+            $req->response( $self->backLogout($req) );
+            return PE_SENDRESPONSE;
+        }
+        elsif ( $path eq $self->conf->{oidcServiceMetaDataFrontChannelURI} ) {
+            $req->response( $self->searchAndLogout($req) );
+            return PE_SENDRESPONSE;
+        }
+    }
 
     # Check callback
     if ( $req->param( $self->conf->{oidcRPCallbackGetParam} ) ) {
@@ -183,8 +203,7 @@ sub extractFormInfo {
             "Refresh token: " . ( $refresh_token || "<none>" ) );
 
         # Verify JWT signature
-        if ( $self->conf->{oidcOPMetaDataOptions}->{$op}
-            ->{oidcOPMetaDataOptionsCheckJWTSignature} )
+        if ( $self->opOptions->{$op}->{oidcOPMetaDataOptionsCheckJWTSignature} )
         {
             unless ( $self->verifyJWTSignature( $id_token, $op ) ) {
                 $self->logger->error("JWT signature verification failed");
@@ -237,14 +256,38 @@ sub extractFormInfo {
             $self->logger->debug('ID Token is valid');
         }
 
-        # Get user id defined in 'sub' field
-        my $user_id = $id_token_payload_hash->{sub};
+        # Get sub from ID token
+        my $id_token_sub = $id_token_payload_hash->{sub};
+        if ( !$id_token_sub ) {
+            $self->logger->error("Could not find sub claim in ID Token");
+            return PE_OIDC_AUTH_ERROR;
+        }
+        my $user_id = $id_token_sub;
+
+        # Get user identifier, if different from sub
+        my $user_claim = $self->conf->{oidcOPMetaDataOptions}->{$op}
+          ->{oidcOPMetaDataOptionsUserAttribute};
+        if ($user_claim) {
+            $user_id = $id_token_payload_hash->{$user_claim};
+            if ( !$user_id ) {
+                $self->logger->error(
+                    "Could not find $user_claim claim in ID Token");
+                return PE_OIDC_AUTH_ERROR;
+            }
+        }
+
+        # Remember ID Token sub
+        $req->data->{id_token_sub} = $id_token_sub;
 
         # Remember tokens
         $req->data->{access_token}  = $access_token;
         $req->data->{refresh_token} = $refresh_token if $refresh_token;
         $req->data->{id_token}      = $id_token;
 
+        # Remember sid claim if given
+        $req->data->{op_sid} = $id_token_payload_hash->{sid}
+          if $id_token_payload_hash->{sid};
+
         # If access token TTL is given save expiration date
         # (with security margin)
         if ($expires_in) {
@@ -300,9 +343,7 @@ sub extractFormInfo {
     $self->logger->debug("Build OpenIDConnect AuthN Request");
 
     my $nonce;
-    if ( $self->conf->{oidcOPMetaDataOptions}->{$op}
-        ->{oidcOPMetaDataOptionsUseNonce} )
-    {
+    if ( $self->opOptions->{$op}->{oidcOPMetaDataOptionsUseNonce} ) {
         $nonce = $self->generateNonce();
         $req->data->{_oidcNonce} = $nonce;
     }
@@ -351,9 +392,17 @@ sub setAuthSessionInfo {
           $req->data->{access_token_eol};
     }
 
+    if ( $req->data->{op_sid} ) {
+        $req->{sessionInfo}->{_oidc_sid} = $req->data->{op_sid};
+    }
+
+    if ( $req->data->{id_token_sub} ) {
+        $req->{sessionInfo}->{_oidc_sub} = $req->data->{id_token_sub};
+    }
+
     # Keep ID Token in session
-    my $store_IDToken = $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsStoreIDToken};
+    my $store_IDToken =
+      $self->opOptions->{$op}->{oidcOPMetaDataOptionsStoreIDToken};
     if ($store_IDToken) {
         $self->logger->debug("Store ID Token in session");
         $req->{sessionInfo}->{_oidc_id_token} = $req->data->{id_token};
@@ -368,15 +417,15 @@ sub setAuthSessionInfo {
 sub authLogout {
     my ( $self, $req ) = @_;
 
+    return PE_OK if $req->data->{oidcSkipLogout};
+
     my $op = $req->{sessionInfo}->{_oidc_OP};
 
     # Find endession endpoint
     my $endsession_endpoint =
-      $self->oidcOPList->{$op}->{conf}->{end_session_endpoint};
+      $self->opMetadata->{$op}->{conf}->{end_session_endpoint};
 
-    my $client_id =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsClientID};
+    my $client_id = $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientID};
 
     if ($endsession_endpoint) {
         my $logout_url = $self->conf->{portal} . '?logout=1';
@@ -400,32 +449,221 @@ sub getDisplayType {
     return "logo";
 }
 
-sub alreadyLoggedOut {
+sub searchAndLogout {
     my ( $self, $req ) = @_;
     $self->userLogger->info(
-        'Front-channel logout request for an already logged out user');
-    my $img = $self->conf->{staticPrefix} . '/common/icons/ok.png';
-
-    # No need to protect this frame
-    $req->frame(1);
-    my $frame = qq'<html><body><img src="$img"></body></html>';
-    return [
-        200,
-        [ 'Content-Type' => 'text/html', 'Content-Length' => length($frame) ],
-        [$frame]
-    ];
+        'Front-channel logout request called without cookie');
+    $req->data->{frontLogoutSearch} = 1;
+    return $self->frontLogout($req);
 }
 
 sub frontLogout {
     my ( $self, $req ) = @_;
-    return $self->p->sendError( $req,
-        'Front Channel logout is not implemented', 501 );
+
+    # No double disconnect
+    $req->data->{oidcSkipLogout} = 1;
+
+    # Response should be displayed in an iframe
+    $req->frame(1);
+
+    # TODO check iss and sid
+    my $iss        = $req->param('iss');
+    my $sid        = $req->param('sid');
+    my $badRequest = 0;
+    my $logName    = 'OIDC FC Logout';
+
+    my $img      = $self->conf->{staticPrefix} . '/common/icons/ok.png';
+    my $frame    = qq'<html><body><img src="$img"></body></html>';
+    my $response = [
+        200,
+        [
+            'Content-Type'   => 'text/html',
+            'Content-Length' => length($frame)
+        ],
+        [$frame]
+    ];
+
+    # LLNG requires sid and iss
+    unless ( $iss and $sid ) {
+        $self->userLogger->error(
+            'OIDC Front-Channel-Logout called without sid and iss');
+        $badRequest++;
+    }
+    elsif ( $req->data->{frontLogoutSearch} ) {
+        my %store = %{ $self->conf->{globalStorageOptions} };
+        $store{backend} = $self->conf->{globalStorage};
+        my $oidcSessions =
+          Lemonldap::NG::Common::Apache::Session->searchOn( \%store,
+            '_oidc_sid', $sid );
+        my @sessionToDelete = keys %$oidcSessions;
+        foreach (@sessionToDelete) {
+            if ( my $as = $self->p->getApacheSession($_) ) {
+                if ( $self->p->_deleteSession( $req, $as, 1 ) ) {
+                    $self->logger->debug(
+                        "$logName: session $_ deleted from global storage");
+                }
+                else {
+                    $self->logger->error(
+                        "$logName: unable to delete session $_");
+                    $self->logger->error( $as->error );
+                }
+            }
+        }
+        return $response;
+    }
+    else {
+        unless ( $req->userData->{_oidc_sid} and $req->userData->{_oidc_OP} ) {
+            $self->logger->error(
+                "$logName: key _oidc_sid not stored in session");
+            $badRequest++;
+        }
+        else {
+            if ( $sid ne $req->userData->{_oidc_sid} ) {
+                $self->userLogger->error(
+                    'OIDC Front-Channel-Logout: sid mismatch');
+                $badRequest++;
+            }
+        }
+    }
+    if ($badRequest) {
+        return $self->p->sendError( $req, 'Bad OIDC Logout Request', 200 );
+    }
+    return $self->p->do(
+        $req,
+        [
+            @{ $self->p->beforeLogout },
+            'authLogout',
+            'deleteSession',
+            sub {
+                $req->response($response);
+                return PE_SENDRESPONSE;
+            }
+        ]
+    );
 }
 
+# Implements https://openid.net/specs/openid-connect-backchannel-1_0.html
 sub backLogout {
     my ( $self, $req ) = @_;
-    return $self->p->sendError( $req, 'Back Channel logout is not implemented',
-        501 );
+    my $logName = 'OIDC BC Logout';
+
+    # REQUIRED CHECKS
+
+    # Back channel requires POST
+    return $self->p->sendError( $req, 'Only POST allowed', 400 )
+      if $req->method ne 'POST';
+
+    # Back channel requires logout_token
+    my $content = $req->body_parameters;
+    return $self->p->sendError( $req, 'logout_token not found', 400 )
+      unless $content and $content->{logout_token};
+
+    # Decode token
+    my $logoutToken = $content->{logout_token};
+    my $payload     = getJWTPayload($logoutToken)
+      or
+      return $self->p->sendError( $req, 'Could not decode logout token', 400 );
+
+    # TODO: validate "alg" header => prohibit "none" value
+
+    # Required fields
+    foreach (qw(iss aud iat jti events)) {
+        return $self->p->sendError( $req, "Missing $_", 400 )
+          unless $payload->{$_};
+    }
+    return $self->p->sendError( $req, 'Missing sub/sid', 400 )
+      unless $payload->{sub} or $payload->{sid};
+
+    # Get iss and found related OP
+    my $op;
+    foreach ( keys %{ $self->opMetadata } ) {
+        if ( $self->opMetadata->{$_}->{conf}->{issuer} eq $payload->{iss} ) {
+            $op = $_;
+            last;
+        }
+    }
+    return $self->p->sendError( $req, 'Issuer not found', 400 ) unless $op;
+
+    # Verify signature
+    return $self->p->sendError( $req, 'Bad signature', 400 )
+      unless $self->verifyJWTSignature( $logoutToken, $op );
+
+    # Verify audience
+    return $self->p->error( $req, 'Bad aud', 400 )
+      unless ref $payload->{aud} eq 'ARRAY';
+    return $self->p->error( $req, 'Client ID not found in audience array', 400 )
+      unless
+      grep { $_ eq $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientID} }
+      @{ $payload->{aud} };
+
+    # Verify iat
+    return $self->p->error( $req, 'Token too old', 400 )
+      unless $payload->{iat} +
+      ( $self->opOptions->{$op}->{oidcOPMetaDataOptionsIDTokenMaxAge} // 30 ) >
+      time;
+
+    # Verify events
+    return $self->p->error( $req, 'Bad events', 400 )
+      unless ref $payload->{events} eq 'HASH'
+      and $payload->{events}->{ $self->BACKCHANNEL_EVENTSKEY() }
+      and ref $payload->{events}->{ $self->BACKCHANNEL_EVENTSKEY() } eq 'HASH';
+
+    # Verify nonce
+    return $self->p->error( $req, 'Bad request', 400 ) if $payload->{nonce};
+
+    # Now request is valid, even if session isn't found, response will be 200
+
+    # Find session
+    my $uid   = $payload->{sub};
+    my $sid   = $payload->{sid};
+    my %store = %{ $self->conf->{globalStorageOptions} };
+    $store{backend} = $self->conf->{globalStorage};
+    my @userSessions;
+    if ($sid) {
+        my $oidcSessions =
+          Lemonldap::NG::Common::Apache::Session->searchOn( \%store,
+            '_oidc_sid', $sid );
+        @userSessions = keys %$oidcSessions;
+    }
+    else {
+        my $oidcSessions =
+          Lemonldap::NG::Common::Apache::Session->searchOn( \%store,
+            '_oidc_sub', $uid );
+        @userSessions =
+          grep {
+                  $oidcSessions->{$_}->{_oidc_OP}
+              and $oidcSessions->{$_}->{_oidc_OP} eq $op;
+          } keys %$oidcSessions;
+    }
+    if (@userSessions) {
+        if ($#userSessions) {
+            $self->logger->warn(
+"$logName: more than one link found between $uid and $op, cleanong all"
+            );
+        }
+        foreach (@userSessions) {
+            if ( my $as = $self->p->getApacheSession($_) ) {
+                if ( $self->p->_deleteSession( $req, $as, 1 ) ) {
+                    $self->logger->debug(
+                        "$logName: session $_ deleted from global storage");
+                }
+                else {
+                    $self->logger->error(
+                        "$logName: unable to delete session $_");
+                    $self->logger->error( $as->error );
+                }
+            }
+            else {
+                $self->logger->info(
+                    "$logName: session $_ for user $uid already deleted");
+            }
+        }
+    }
+    else {
+        $self->logger->warn("$logName: no session found for $uid");
+    }
+
+    return [ 200, [], [] ];
 }
 
 1;
