@@ -28,7 +28,10 @@ use URI;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT PE_ERROR);
 
-our $VERSION = '2.16.3';
+our $VERSION = '2.17.0';
+
+use constant BACKCHANNEL_EVENTSKEY =>
+  'http://schemas.openid.net/event/backchannel-logout';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -40,7 +43,13 @@ use constant ADDRESS =>
   [qw/formatted street_address locality region postal_code country/];
 use constant PHONE => [qw/phone_number phone_number_verified/];
 
-use constant OIDC_SCOPES => [qw/openid profile email address phone/];
+use constant DEFAULT_SCOPES => {
+    profile => PROFILE,
+    email   => EMAIL,
+    address => ADDRESS,
+    phone   => PHONE,
+};
+
 use constant COMPLEX_CLAIM => {
     formatted      => "address",
     street_address => "address",
@@ -52,13 +61,23 @@ use constant COMPLEX_CLAIM => {
 
 # PROPERTIES
 
-has oidcOPList   => ( is => 'rw', default => sub { {} }, );
-has oidcRPList   => ( is => 'rw', default => sub { {} }, );
-has rpAttributes => ( is => 'rw', default => sub { {} }, );
+has opAttributes => ( is => 'rw', default => sub { {} } );
+has opMetadata   => ( is => 'rw', default => sub { {} }, );
+has opOptions    => ( is => 'rw', default => sub { {} }, );
 has opRules      => ( is => 'rw', default => sub { {} } );
-has spRules      => ( is => 'rw', default => sub { {} } );
-has spMacros     => ( is => 'rw', default => sub { {} } );
-has spScopeRules => ( is => 'rw', default => sub { {} } );
+has rpAttributes => ( is => 'rw', default => sub { {} }, );
+has rpMacros     => ( is => 'rw', default => sub { {} } );
+has rpOptions    => ( is => 'rw', default => sub { {} }, );
+has rpRules      => ( is => 'rw', default => sub { {} } );
+has rpScopes     => ( is => 'rw', default => sub { {} } );
+has rpScopeRules => ( is => 'rw', default => sub { {} } );
+
+# Deprecated names, remove in 3.0
+*oidcOPList   = *opMetadata;
+*oidcRPList   = *rpOptions;
+*spMacros     = *rpMacros;
+*spRules      = *rpRules;
+*spScopeRules = *rpScopeRules;
 
 # return LWP::UserAgent object
 has ua => (
@@ -105,8 +124,8 @@ sub loadOPs {
         my $op_conf =
           $self->decodeJSON( $self->conf->{oidcOPMetaDataJSON}->{$_} );
         if ($op_conf) {
-            $self->oidcOPList->{$_}->{conf} = $op_conf;
-            $self->oidcOPList->{$_}->{jwks} =
+            $self->opMetadata->{$_}->{conf} = $op_conf;
+            $self->opMetadata->{$_}->{jwks} =
               $self->decodeJSON( $self->conf->{oidcOPMetaDataJWKS}->{$_} );
         }
         else {
@@ -116,8 +135,11 @@ sub loadOPs {
 
     # Set rule
     foreach ( keys %{ $self->conf->{oidcOPMetaDataOptions} } ) {
-        my $cond = $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsResolutionRule};
+        $self->opAttributes->{$_} =
+          $self->conf->{oidcOPMetaDataExportedVars}->{$_};
+        $self->opOptions->{$_} = $self->conf->{oidcOPMetaDataOptions}->{$_};
+        my $cond =
+          $self->opOptions->{$_}->{oidcOPMetaDataOptionsResolutionRule};
         if ( length $cond ) {
             my $rule_sub =
               $self->p->buildRule( $cond, "OIDC provider resolution" );
@@ -131,11 +153,14 @@ sub loadOPs {
 }
 
 # Load OpenID Connect Relying Parties
+# DEPRECATED, remove in 3.0, use sendOidcResponse instead
 # @param no_cache Disable cache use
 # @return boolean result
 sub loadRPs {
     my ($self) = @_;
 
+    $self->warning->info("Using loadRPs is deprecated, use getRP instead");
+
     # Check presence of at least one relying party in configuration
     unless ( $self->conf->{oidcRPMetaDataOptions}
         and keys %{ $self->conf->{oidcRPMetaDataOptions} } )
@@ -146,93 +171,108 @@ sub loadRPs {
     }
 
     foreach my $rp ( keys %{ $self->conf->{oidcRPMetaDataOptions} || {} } ) {
-        my $valid = 1;
+        $self->load_rp_from_llng_conf($rp);
+    }
+}
 
-        # Handle attributes
-        my $attributes = {
-            profile => PROFILE,
-            email   => EMAIL,
-            address => ADDRESS,
-            phone   => PHONE,
-        };
+# Load a single RP from LLNG configuration
+sub load_rp_from_llng_conf {
+    my ( $self, $rp ) = @_;
 
-        # Additional claims
-        my $extraClaims =
-          $self->conf->{oidcRPMetaDataOptionsExtraClaims}->{$rp};
+    return $self->load_rp(
+        confKey     => $rp,
+        extraClaims => $self->conf->{oidcRPMetaDataOptionsExtraClaims}->{$rp},
+        options     => $self->conf->{oidcRPMetaDataOptions}->{$rp},
+        macros      => $self->conf->{oidcRPMetaDataMacros}->{$rp},
+        scopeRules  => $self->conf->{oidcRPMetaDataScopeRules}->{$rp},
+        attributes  => $self->conf->{oidcRPMetaDataExportedVars}->{$rp},
+    );
+}
 
-        if ($extraClaims) {
-            foreach my $claim ( keys %$extraClaims ) {
-                $self->logger->debug("Using extra claim $claim for $rp");
-                my @extraAttributes = split( /\s/, $extraClaims->{$claim} );
-                $attributes->{$claim} = \@extraAttributes;
-            }
+sub load_rp {
+    my ( $self, %config ) = @_;
+    my $rp = $config{confKey};
+
+    my $valid = 1;
+
+    # Handle scopes
+    # this HAS to be a deep copy of the DEFAULT_SCOPES hashref!
+    my $scope_values = { %{ DEFAULT_SCOPES() } };
+
+    # Additional claims
+    my $extraClaims = $config{extraClaims};
+
+    if ($extraClaims) {
+        foreach my $scope ( keys %$extraClaims ) {
+            $self->logger->debug("Processing scope value $scope for $rp");
+            my @extraAttributes = split( /\s/, $extraClaims->{$scope} );
+            $scope_values->{$scope} = \@extraAttributes;
         }
+    }
 
-        # Access rule
-        my $rule = $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsRule};
-        if ( length $rule ) {
-            $rule = $self->p->HANDLER->substitute($rule);
-            unless ( $rule = $self->p->HANDLER->buildSub($rule) ) {
-                $self->logger->error( "Unable to build access rule for RP $rp: "
+    # Access rule
+    my $rule = $config{options}->{oidcRPMetaDataOptionsRule};
+    if ( length $rule ) {
+        $rule = $self->p->HANDLER->substitute($rule);
+        unless ( $rule = $self->p->HANDLER->buildSub($rule) ) {
+            $self->logger->error( "Unable to build access rule for RP $rp: "
+                  . $self->p->HANDLER->tsv->{jail}->error );
+            $valid = 0;
+        }
+    }
+
+    # Load per-RP macros
+    my $macros         = $config{macros};
+    my $compiledMacros = {};
+    for my $macroAttr ( keys %{$macros} ) {
+        my $macroRule = $macros->{$macroAttr};
+        if ( length $macroRule ) {
+            $macroRule = $self->p->HANDLER->substitute($macroRule);
+            if ( $macroRule = $self->p->HANDLER->buildSub($macroRule) ) {
+                $compiledMacros->{$macroAttr} = $macroRule;
+            }
+            else {
+                $self->logger->error(
+                    "Unable to build macro $macroAttr for RP $rp:"
                       . $self->p->HANDLER->tsv->{jail}->error );
                 $valid = 0;
             }
         }
+    }
 
-        # Load per-RP macros
-        my $macros         = $self->conf->{oidcRPMetaDataMacros}->{$rp};
-        my $compiledMacros = {};
-        for my $macroAttr ( keys %{$macros} ) {
-            my $macroRule = $macros->{$macroAttr};
-            if ( length $macroRule ) {
-                $macroRule = $self->p->HANDLER->substitute($macroRule);
-                if ( $macroRule = $self->p->HANDLER->buildSub($macroRule) ) {
-                    $compiledMacros->{$macroAttr} = $macroRule;
-                }
-                else {
-                    $self->logger->error(
-                        "Unable to build macro $macroAttr for RP $rp:"
-                          . $self->p->HANDLER->tsv->{jail}->error );
-                    $valid = 0;
-                }
+    # Load per-RP dynamic scopes
+    my $scope_rules          = $config{scopeRules};
+    my $compiled_scope_rules = {};
+    for my $scopeName ( keys %{$scope_rules} ) {
+        my $scopeRule = $scope_rules->{$scopeName};
+        if ( length $scopeRule ) {
+            $scopeRule = $self->p->HANDLER->substitute($scopeRule);
+            if ( $scopeRule = $self->p->HANDLER->buildSub($scopeRule) ) {
+                $compiled_scope_rules->{$scopeName} = $scopeRule;
             }
-        }
-
-        # Load per-RP dynamic scopes
-        my $scopes         = $self->conf->{oidcRPMetaDataScopeRules}->{$rp};
-        my $compiledScopes = {};
-        for my $scopeName ( keys %{$scopes} ) {
-            my $scopeRule = $scopes->{$scopeName};
-            if ( length $scopeRule ) {
-                $scopeRule = $self->p->HANDLER->substitute($scopeRule);
-                if ( $scopeRule = $self->p->HANDLER->buildSub($scopeRule) ) {
-                    $compiledScopes->{$scopeName} = $scopeRule;
-                }
-                else {
-                    $self->logger->error(
-                        "Unable to build scope $scopeName for RP $rp:"
-                          . $self->p->HANDLER->tsv->{jail}->error );
-                    $valid = 0;
-                }
+            else {
+                $self->logger->error(
+                    "Unable to build scope $scopeName for RP $rp:"
+                      . $self->p->HANDLER->tsv->{jail}->error );
+                $valid = 0;
             }
         }
-        if ($valid) {
-
-            # Register RP
-            $self->oidcRPList->{$rp} =
-              $self->conf->{oidcRPMetaDataOptions}->{$rp};
-            $self->rpAttributes->{$rp} = $attributes;
-            $self->spMacros->{$rp}     = $compiledMacros;
-            $self->spScopeRules->{$rp} = $compiledScopes;
-            $self->spRules->{$rp}      = $rule;
-        }
-        else {
-            $self->logger->error(
-                "Relaying Party $rp has errors and will be ignored");
-        }
     }
-    return 1;
+    if ($valid) {
+
+        # Register RP
+        $self->rpOptions->{$rp}    = $config{options};
+        $self->rpAttributes->{$rp} = $config{attributes};
+        $self->rpScopes->{$rp}     = $scope_values;
+        $self->rpMacros->{$rp}     = $compiledMacros;
+        $self->rpScopeRules->{$rp} = $compiled_scope_rules;
+        $self->rpRules->{$rp}      = $rule;
+    }
+    else {
+        $self->logger->error(
+            "Relaying Party $rp has errors and will be ignored");
+    }
+    return;
 }
 
 # Refresh JWKS data if needed
@@ -256,9 +296,8 @@ sub refreshJWKSdata {
         # 2/ jwks_uri defined in metadata
 
         my $jwksTimeout =
-          $self->conf->{oidcOPMetaDataOptions}->{$_}
-          ->{oidcOPMetaDataOptionsJWKSTimeout};
-        my $jwksUri = $self->oidcOPList->{$_}->{conf}->{jwks_uri};
+          $self->opOptions->{$_}->{oidcOPMetaDataOptionsJWKSTimeout};
+        my $jwksUri = $self->opMetadata->{$_}->{conf}->{jwks_uri};
 
         unless ($jwksTimeout) {
             $self->logger->debug(
@@ -271,7 +310,7 @@ sub refreshJWKSdata {
             next;
         }
 
-        if ( $self->oidcOPList->{$_}->{jwks}->{time} + $jwksTimeout > time ) {
+        if ( $self->opMetadata->{$_}->{jwks}->{time} + $jwksTimeout > time ) {
             $self->logger->debug("JWKS data still valid for $_, skipping...");
             next;
         }
@@ -290,31 +329,13 @@ sub refreshJWKSdata {
 
         my $content = $self->decodeJSON( $response->decoded_content );
 
-        $self->oidcOPList->{$_}->{jwks} = $content;
-        $self->oidcOPList->{$_}->{jwks}->{time} = time;
+        $self->opMetadata->{$_}->{jwks} = $content;
+        $self->opMetadata->{$_}->{jwks}->{time} = time;
 
     }
     return 1;
 }
 
-# Get Relying Party corresponding to a Client ID
-# @param client_id Client ID
-# @return String result
-sub getRP {
-    my ( $self, $client_id ) = @_;
-    my $rp;
-
-    foreach ( keys %{ $self->oidcRPList } ) {
-        if ( $client_id eq
-            $self->oidcRPList->{$_}->{oidcRPMetaDataOptionsClientID} )
-        {
-            $rp = $_;
-            last;
-        }
-    }
-    return $rp;
-}
-
 # Compute callback URI
 # @return String Callback URI
 sub getCallbackUri {
@@ -340,7 +361,7 @@ sub buildAuthorizationCodeAuthnRequest {
     my ( $self, $req, $op, $state, $nonce ) = @_;
 
     my $authorize_uri =
-      $self->oidcOPList->{$op}->{conf}->{authorization_endpoint};
+      $self->opMetadata->{$op}->{conf}->{authorization_endpoint};
 
     unless ($authorize_uri) {
         $self->logger->error(
@@ -349,28 +370,15 @@ sub buildAuthorizationCodeAuthnRequest {
         );
         return undef;
     }
-    my $client_id =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsClientID};
-    my $scope =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}->{oidcOPMetaDataOptionsScope};
+    my $client_id = $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientID};
+    my $scope     = $self->opOptions->{$op}->{oidcOPMetaDataOptionsScope};
     my $response_type = "code";
     my $redirect_uri  = $self->getCallbackUri($req);
-    my $display =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsDisplay};
-    my $prompt =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsPrompt};
-    my $max_age =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsMaxAge};
-    my $ui_locales =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsUiLocales};
-    my $acr_values =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsAcrValues};
+    my $display       = $self->opOptions->{$op}->{oidcOPMetaDataOptionsDisplay};
+    my $prompt        = $self->opOptions->{$op}->{oidcOPMetaDataOptionsPrompt};
+    my $max_age       = $self->opOptions->{$op}->{oidcOPMetaDataOptionsMaxAge};
+    my $ui_locales = $self->opOptions->{$op}->{oidcOPMetaDataOptionsUiLocales};
+    my $acr_values = $self->opOptions->{$op}->{oidcOPMetaDataOptionsAcrValues};
 
     my $authorize_request_params = {
         response_type => $response_type,
@@ -608,14 +616,11 @@ sub getAccessTokenFromTokenEndpoint {
 
     $grant_options ||= {};
 
-    my $client_id =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsClientID};
+    my $client_id = $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientID};
     my $client_secret =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsClientSecret};
+      $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientSecret};
     my $access_token_uri =
-      $self->oidcOPList->{$op}->{conf}->{token_endpoint};
+      $self->opMetadata->{$op}->{conf}->{token_endpoint};
 
     unless ($access_token_uri) {
         $self->logger->error(
@@ -626,8 +631,7 @@ sub getAccessTokenFromTokenEndpoint {
     }
 
     my $auth_method =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsTokenEndpointAuthMethod}
+      $self->opOptions->{$op}->{oidcOPMetaDataOptionsTokenEndpointAuthMethod}
       || 'client_secret_post';
 
     unless ( $auth_method =~ /^client_secret_(basic|post)$/o ) {
@@ -718,24 +722,15 @@ sub checkTokenResponseValidity {
 sub checkIDTokenValidity {
     my ( $self, $op, $id_token, $state_nonce ) = @_;
 
-    my $client_id =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsClientID};
-    my $acr_values =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsAcrValues};
-    my $max_age =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsMaxAge};
+    my $client_id  = $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientID};
+    my $acr_values = $self->opOptions->{$op}->{oidcOPMetaDataOptionsAcrValues};
+    my $max_age    = $self->opOptions->{$op}->{oidcOPMetaDataOptionsMaxAge};
     my $id_token_max_age =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsIDTokenMaxAge};
-    my $use_nonce =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsUseNonce};
+      $self->opOptions->{$op}->{oidcOPMetaDataOptionsIDTokenMaxAge};
+    my $use_nonce = $self->opOptions->{$op}->{oidcOPMetaDataOptionsUseNonce};
 
     # Check issuer
-    unless ( $id_token->{iss} eq $self->oidcOPList->{$op}->{conf}->{issuer} ) {
+    unless ( $id_token->{iss} eq $self->opMetadata->{$op}->{conf}->{issuer} ) {
         $self->logger->error("Issuer mismatch");
         return 0;
     }
@@ -743,7 +738,7 @@ sub checkIDTokenValidity {
     # Check audience
     if ( ref $id_token->{aud} ) {
         my @audience = @{ $id_token->{aud} };
-        unless ( grep $_ eq $client_id, @audience ) {
+        unless ( grep { $_ eq $client_id } @audience ) {
             $self->logger->error("Client ID not found in audience array");
             return 0;
         }
@@ -963,7 +958,7 @@ sub getUserInfo {
     my ( $self, $op, $access_token ) = @_;
 
     my $userinfo_uri =
-      $self->oidcOPList->{$op}->{conf}->{userinfo_endpoint};
+      $self->opMetadata->{$op}->{conf}->{userinfo_endpoint};
 
     unless ($userinfo_uri) {
         $self->logger->error("UserInfo URI not found in $op configuration");
@@ -1004,6 +999,10 @@ sub decodeJSON {
 
     eval { $json_hash = from_json( $json, { allow_nonref => 1 } ); };
     return undef if ($@);
+    unless ( ref $json_hash ) {
+        $self->logger->error("Wanted a JSON object, got: $json_hash");
+        return undef;
+    }
 
     return $json_hash;
 }
@@ -1030,7 +1029,7 @@ sub newAuthorizationCode {
     return $self->getOpenIDConnectSession(
         undef,
         "authorization_code",
-        $self->conf->{oidcRPMetaDataOptions}->{$rp}
+        $self->rpOptions->{$rp}
           ->{oidcRPMetaDataOptionsAuthorizationCodeExpiration}
           || $self->conf->{oidcServiceAuthorizationCodeExpiration},
         ,
@@ -1069,8 +1068,7 @@ sub newAccessToken {
     my $session = $self->getOpenIDConnectSession(
         undef,
         "access_token",
-        $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsAccessTokenExpiration}
+        $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAccessTokenExpiration}
           || $self->conf->{oidcServiceAccessTokenExpiration},
         $at_info,
     );
@@ -1094,19 +1092,17 @@ sub newAccessToken {
 
 sub _wantJWT {
     my ( $self, $rp ) = @_;
-    return $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsAccessTokenJWT};
+    return $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAccessTokenJWT};
 }
 
 sub makeJWT {
     my ( $self, $req, $rp, $scope, $id, $sessionInfo ) = @_;
 
     my $exp =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsAccessTokenExpiration}
+         $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAccessTokenExpiration}
       || $self->conf->{oidcServiceAccessTokenExpiration};
     $exp += time;
-    my $client_id = $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsClientID};
+    my $client_id = $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientID};
 
     my $access_token_payload = {
         iss       => $self->iss,                  # Issuer Identifier
@@ -1116,17 +1112,17 @@ sub makeJWT {
         iat       => time,                        # Issued time
         jti       => $id,                         # Access Token session ID
         scope     => $scope,                      # Scope
+        sid       => $self->getSidFromSession( $rp, $sessionInfo ), # Session id
     };
 
     my $claims =
       $self->buildUserInfoResponseFromData( $req, $scope, $rp, $sessionInfo );
 
     # Release claims, or only sub
-    if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
-        ->{oidcRPMetaDataOptionsAccessTokenClaims} )
-    {
+    if ( $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAccessTokenClaims} ) {
         foreach ( keys %$claims ) {
-            $access_token_payload->{$_} = $claims->{$_};
+            $access_token_payload->{$_} = $claims->{$_}
+              unless $access_token_payload->{$_};
         }
     }
     else {
@@ -1139,8 +1135,8 @@ sub makeJWT {
     return undef if ( $h != PE_OK );
 
     # Get signature algorithm
-    my $alg = $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsAccessTokenSignAlg} || "RS256";
+    my $alg = $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAccessTokenSignAlg}
+      || "RS256";
     $self->logger->debug("Access Token signature algorithm: $alg");
 
     my $jwt = $self->createJWT( $access_token_payload, $alg, $rp, "at+JWT" );
@@ -1187,8 +1183,8 @@ sub newRefreshToken {
     my ( $self, $rp, $info, $offline ) = @_;
     my $ttl =
       $offline
-      ? ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsOfflineSessionExpiration}
+      ? (
+        $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsOfflineSessionExpiration}
           || $self->conf->{oidcServiceOfflineSessionExpiration} )
       : $self->conf->{timeout};
 
@@ -1214,20 +1210,8 @@ sub updateRefreshToken {
 sub updateToken {
     my ( $self, $id, $infos ) = @_;
 
-    my %storage = (
-        storageModule        => $self->conf->{oidcStorage},
-        storageModuleOptions => $self->conf->{oidcStorageOptions},
-    );
-
-    unless ( $storage{storageModule} ) {
-        %storage = (
-            storageModule        => $self->conf->{globalStorage},
-            storageModuleOptions => $self->conf->{globalStorageOptions},
-        );
-    }
-
     my $oidcSession = Lemonldap::NG::Common::Session->new( {
-            %storage,
+            $self->_storeOpts(),
             cacheModule        => $self->conf->{localSessionStorage},
             cacheModuleOptions => $self->conf->{localSessionStorageOptions},
             id                 => $id,
@@ -1249,22 +1233,11 @@ sub updateToken {
 # @return Lemonldap::NG::Common::Session object
 sub getOpenIDConnectSession {
     my ( $self, $id, $type, $ttl, $info ) = @_;
-    my %storage = (
-        storageModule        => $self->conf->{oidcStorage},
-        storageModuleOptions => $self->conf->{oidcStorageOptions},
-    );
 
     $ttl ||= $self->conf->{timeout};
 
-    unless ( $storage{storageModule} ) {
-        %storage = (
-            storageModule        => $self->conf->{globalStorage},
-            storageModuleOptions => $self->conf->{globalStorageOptions},
-        );
-    }
-
     my $oidcSession = Lemonldap::NG::Common::Session->new( {
-            %storage,
+            $self->_storeOpts(),
             cacheModule        => $self->conf->{localSessionStorage},
             cacheModuleOptions => $self->conf->{localSessionStorageOptions},
             id                 => $id,
@@ -1348,7 +1321,7 @@ sub storeState {
     return $self->state_ott->createToken($infos);
 }
 
-# Extract state information into $self
+# Extract state information into $req
 sub extractState {
     my ( $self, $req, $state ) = @_;
 
@@ -1414,12 +1387,10 @@ sub verifyJWTSignature {
         # Check signature with client secret
         my $client_secret;
         $client_secret =
-          $self->conf->{oidcOPMetaDataOptions}->{$op}
-          ->{oidcOPMetaDataOptionsClientSecret}
+          $self->opOptions->{$op}->{oidcOPMetaDataOptionsClientSecret}
           if ($op);
         $client_secret =
-          $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsClientSecret}
+          $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientSecret}
           if ($rp);
 
         my $digest;
@@ -1456,13 +1427,13 @@ sub verifyJWTSignature {
         }
 
         # The public key is needed
-        unless ( $self->oidcOPList->{$op}->{jwks} ) {
+        unless ( $self->opMetadata->{$op}->{jwks} ) {
             $self->logger->error(
                 "Cannot verify $alg signature: no JWKS data found");
             return 0;
         }
 
-        my $keys = $self->oidcOPList->{$op}->{jwks}->{keys};
+        my $keys = $self->opMetadata->{$op}->{jwks}->{keys};
         my $key_hash;
 
         # Find Key ID associated with signature
@@ -1711,9 +1682,7 @@ sub checkEndPointAuthenticationCredentials {
     }
 
     # Check client_secret
-    if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
-        ->{oidcRPMetaDataOptionsPublic} )
-    {
+    if ( $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsPublic} ) {
         $self->logger->debug(
             "Relying Party $rp is public, do not check client secret");
     }
@@ -1724,8 +1693,8 @@ sub checkEndPointAuthenticationCredentials {
             );
             return undef;
         }
-        unless ( $client_secret eq $self->conf->{oidcRPMetaDataOptions}->{$rp}
-            ->{oidcRPMetaDataOptionsClientSecret} )
+        unless ( $client_secret eq
+            $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientSecret} )
         {
             $self->logger->error("Wrong credentials for $rp");
             return undef;
@@ -1798,7 +1767,7 @@ sub getAttributesListFromClaim {
 # @return arrayref attributes list
 sub getAttributesListFromScopeValue {
     my ( $self, $rp, $scope_value ) = @_;
-    return $self->rpAttributes->{$rp}->{$scope_value};
+    return $self->rpScopes->{$rp}->{$scope_value};
 }
 
 # Return granted scopes for this request
@@ -1813,13 +1782,8 @@ sub getScope {
     # Clean up unknown scopes
     if ( $self->conf->{oidcServiceAllowOnlyDeclaredScopes} ) {
         my @known_scopes = (
-            keys( %{ $self->spScopeRules->{$rp} || {} } ),
-            @{ OIDC_SCOPES() },
-            keys(
-                %{
-                    $self->conf->{oidcRPMetaDataOptionsExtraClaims}->{$rp} || {}
-                }
-            )
+            keys( %{ $self->rpScopeRules->{$rp} || {} } ),
+            "openid", keys( %{ $self->rpScopes->{$rp} || {} } )
         );
         my @scope_values_tmp;
         for my $scope_value (@scope_values) {
@@ -1835,10 +1799,10 @@ sub getScope {
     }
 
     # If this RP has dynamic scopes
-    if ( $self->spScopeRules->{$rp} ) {
+    if ( $self->rpScopeRules->{$rp} ) {
 
         # Add dynamic scopes
-        for my $dynamicScope ( keys %{ $self->spScopeRules->{$rp} } ) {
+        for my $dynamicScope ( keys %{ $self->rpScopeRules->{$rp} } ) {
 
             # Set a magic "$requested" variable that contains true if the
             # scope was requested by the application
@@ -1846,7 +1810,7 @@ sub getScope {
             my $attributes = { %{ $req->userData }, requested => $requested };
 
             # If scope is granted by the rule
-            if ( $self->spScopeRules->{$rp}->{$dynamicScope}
+            if ( $self->rpScopeRules->{$rp}->{$dynamicScope}
                 ->( $req, $attributes ) )
             {
                 # Add to list
@@ -1897,7 +1861,7 @@ sub buildUserInfoResponse {
 sub _addAttributeToResponse {
     my ( $self, $req, $data, $userinfo_response, $rp, $attribute ) = @_;
     my @attrConf = split /;/,
-      ( $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute} || "" );
+      ( $self->rpAttributes->{$rp}->{$attribute} || "" );
     my $session_key = $attrConf[0];
     if ($session_key) {
         my $type  = $attrConf[1] || 'string';
@@ -1906,9 +1870,9 @@ sub _addAttributeToResponse {
         my $session_value;
 
         # Lookup attribute in macros first
-        if ( $self->spMacros->{$rp}->{$session_key} ) {
+        if ( $self->rpMacros->{$rp}->{$session_key} ) {
             $session_value =
-              $self->spMacros->{$rp}->{$session_key}->( $req, $data );
+              $self->rpMacros->{$rp}->{$session_key}->( $req, $data );
 
             # If not found, search in session
         }
@@ -1950,7 +1914,7 @@ sub buildUserInfoResponseFromData {
 
     my $data = {
         %{$session_data},
-        _clientId => $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsClientID},
+        _clientId => $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientID},
         _clientConfKey => $rp,
         _scope         => $scope,
     };
@@ -1962,9 +1926,7 @@ sub buildUserInfoResponseFromData {
 
     # By default, release all exported attributes
     if ( $self->conf->{oidcServiceIgnoreScopeForClaims} ) {
-        for my $attribute (
-            keys %{ $self->conf->{oidcRPMetaDataExportedVars}->{$rp} || {} } )
-        {
+        for my $attribute ( keys %{ $self->rpAttributes->{$rp} || {} } ) {
             $self->_addAttributeToResponse( $req, $data, $userinfo_response,
                 $rp, $attribute );
         }
@@ -2104,8 +2066,7 @@ sub createJWT {
 
         # Sign with client secret
         my $client_secret =
-          $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsClientSecret};
+          $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientSecret};
         unless ($client_secret) {
             $self->logger->error(
                 "Algorithm $alg needs a Client Secret to sign JWT");
@@ -2180,8 +2141,7 @@ sub createIDToken {
     my ( $self, $req, $payload, $rp ) = @_;
 
     # Get signature algorithm
-    my $alg = $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsIDTokenSignAlg};
+    my $alg = $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsIDTokenSignAlg};
     $self->logger->debug("ID Token signature algorithm: $alg");
 
     my $h = $self->p->processHook( $req, 'oidcGenerateIDToken', $payload, $rp );
@@ -2331,6 +2291,11 @@ sub getRequestJWT {
 
 sub addRouteFromConf {
     my ( $self, $type, %subs ) = @_;
+
+    # avoid a warning in logs when route is already defined
+    my $getter = { "Auth" => "authRoutes", "Unauth" => "unAuthRoutes" }->{$type}
+      || "${type}Routes";
+
     my $adder = "add${type}Route";
     foreach ( keys %subs ) {
         my $sub  = $subs{$_};
@@ -2339,6 +2304,10 @@ sub addRouteFromConf {
             $self->logger->error("$_ parameter not defined");
             next;
         }
+
+        # Avoid warning if loading modules twice
+        next if $self->p->$getter->{GET}->{ $self->path }->{$path};
+
         $self->$adder(
             $self->path => { $path => $sub },
             [ 'GET', 'POST' ]
@@ -2354,11 +2323,6 @@ sub addRouteFromConf {
 sub validatePKCEChallenge {
     my ( $self, $code_verifier, $code_challenge, $code_challenge_method ) = @_;
 
-    unless ($code_challenge) {
-        $self->logger->debug("PKCE was not requested by the RP");
-        return 1;
-    }
-
     unless ($code_verifier) {
         $self->logger->error("PKCE required but no code verifier provided");
         return 0;
@@ -2399,8 +2363,7 @@ sub validatePKCEChallenge {
 
 sub force_id_claims {
     my ( $self, $rp ) = @_;
-    return $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsIDTokenForceClaims};
+    return $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsIDTokenForceClaims};
 }
 
 # https://openid.net/specs/openid-connect-core-1_0.html#IDToken
@@ -2412,9 +2375,9 @@ sub force_id_claims {
 sub getAudiences {
     my ( $self, $rp ) = @_;
 
-    my $client_id = $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsClientID};
+    my $client_id    = $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsClientID};
     my @addAudiences = split /\s+/,
-      ( $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsAdditionalAudiences}
+      ( $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsAdditionalAudiences}
           || '' );
 
     my $result = [$client_id];
@@ -2429,22 +2392,43 @@ sub getUserIDForRP {
     my ( $self, $req, $rp, $data ) = @_;
 
     my $user_id_attribute =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsUserIDAttr}
+         $self->rpOptions->{$rp}->{oidcRPMetaDataOptionsUserIDAttr}
       || $self->conf->{whatToTrace};
 
     # If the main attribute is a SP macro, resolve it
     # else, get it directly from session data
-    return $self->spMacros->{$rp}->{$user_id_attribute}
-      ? $self->spMacros->{$rp}->{$user_id_attribute}->( $req, $data )
+    return $self->rpMacros->{$rp}->{$user_id_attribute}
+      ? $self->rpMacros->{$rp}->{$user_id_attribute}->( $req, $data )
       : $data->{$user_id_attribute};
 }
 
+# Return storage options
+sub _storeOpts {
+    my ($self) = @_;
+    my $storage =
+      $self->conf->{oidcStorage}
+      ? {
+        storageModule        => $self->conf->{oidcStorage},
+        storageModuleOptions => $self->conf->{oidcStorageOptions},
+      }
+      : {
+        storageModule        => $self->conf->{globalStorage},
+        storageModuleOptions => $self->conf->{globalStorageOptions},
+      };
+    return %$storage;
+}
+
 sub generateNonce {
     my ($self) = @_;
     return encode_base64url( Crypt::URandom::urandom(16) );
 }
 
+sub getSidFromSession {
+    my ( $self, $rp, $sessionInfo ) = @_;
+    return Digest::SHA::hmac_sha256_base64(
+        $sessionInfo->{_session_id} . ':' . $rp );
+}
+
 1;
 
 __END__
