@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.17.0';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -109,7 +109,8 @@ sub _verify {
 
     my $session;
     unless ( $session = $self->ott->getToken($token) ) {
-        $self->userLogger->info('Token expired');
+        $self->userLogger->info(
+            'Invalid token during ' . $self->prefix . '2f validation' );
         $req->noLoginDisplay(1);
         return $self->p->do( $req, [ sub { PE_TOKENEXPIRED } ] );
     }
@@ -141,7 +142,7 @@ sub _verify {
     # Else restore session
     $req->mustRedirect(1);
     $self->userLogger->notice( $self->prefix
-          . '2f verification for '
+          . '2f verification succeeded for '
           . $req->sessionInfo->{ $self->conf->{whatToTrace} } );
 
     if ( my $l = $self->authnLevel ) {
@@ -201,7 +202,6 @@ sub _verify {
         [
             @{ $self->p->afterData },
             $self->p->validSession,
-            'rebuildCookies',
             @{ $self->p->endAuth },
             sub { PE_OK }
         ]
